@@ -3,13 +3,12 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
 import datetime
-from itertools import chain
-from typing import Tuple, Optional, TYPE_CHECKING, List, cast, Generator
+from typing import Tuple, Optional, TYPE_CHECKING, List, cast, Any
 
 from qt import QPointF, QRectF, QDialogButtonBox
 from qt import QGraphicsRectItem, QGraphicsSimpleTextItem
@@ -21,10 +20,10 @@ from tile import MeldList
 from wind import Wind
 from tilesource import TileSource
 from animation import animate
-from log import logError, logDebug, logWarning, i18n
-from query import Query, openDb
+from log import logException, logDebug, logWarning, i18n
+from query import Query
 from uitile import UITile, UIMeld
-from board import WindLabel, Board
+from board import WindLabel
 from game import Game
 from games import Games
 from hand import Hand
@@ -35,6 +34,7 @@ from tables import SelectRuleset
 from uiwall import UIWall, SideText
 from guiutil import decorateWindow, BlockSignals, rotateCenter, sceneRotation
 from mi18n import i18nc
+from util import callers
 
 if TYPE_CHECKING:
     from qt import QWidget, QEvent, QGraphicsSceneDragDropEvent, QGraphicsItem, QGraphicsItemGroup, QObject
@@ -56,7 +56,7 @@ class SwapDialog(QMessageBox):
     """ask the user if two players should change seats"""
 
     def __init__(self, swappers:List[Player], parent:Optional['QWidget']=None) ->None:
-        QMessageBox.__init__(self, parent)
+        super().__init__(parent)
         decorateWindow(self, i18nc("@title:window", "Swap Seats"))
         self.setText(
             i18n("By the rules, %1 and %2 should now exchange their seats. ",
@@ -80,7 +80,7 @@ class SelectPlayers(SelectRuleset):
     """a dialog for selecting four players. Used only for scoring game."""
 
     def __init__(self) ->None:
-        SelectRuleset.__init__(self)
+        super().__init__()
         Players.load()
         decorateWindow(self, i18nc("@title:window", "Select four players"))
         self.names:List[str]
@@ -95,22 +95,26 @@ class SelectPlayers(SelectRuleset):
             self.grid.addWidget(WindLabel(wind), idx + 1, 0)
             cbName.currentIndexChanged.connect(self.slotValidate)
 
+        # scoring games have seed 0
+        # a scoring game which only just started or threw an exception might have p0,p1,p2,p3 = null
         query = Query(
-            "select p0,p1,p2,p3 from game where seed=0 and game.id = (select max(id) from game)")
+            "select id,p0,p1,p2,p3 from game where seed=0 and p0 is not null order by id desc limit 1")
         if query.records:
+            record = query.record()
+            max_game_id = record[0]
             with BlockSignals(self.nameWidgets):
-                for cbName, playerId in zip(self.nameWidgets, query.records[0]):
-                    try:
-                        playerName = Players.humanNames[playerId]
-                        playerIdx = cbName.findText(playerName)
-                        if playerIdx >= 0:
-                            cbName.setCurrentIndex(playerIdx)
-                    except KeyError:
-                        logError(f'database is inconsistent: player with id {int(playerId)} '
-                                 f'is in game but not in player')
+                for cbName, playerId in zip(self.nameWidgets, record[1:]):
+                    if playerId not in Players.humanNames:
+                        logException(f'database {Internal.db.path} is inconsistent: '
+                                     f'game {max_game_id}: player with id {playerId} '
+                                     f'is in game but not in player')
+                    playerName = Players.humanNames[playerId]
+                    playerIdx = cbName.findText(playerName)
+                    if playerIdx >= 0:
+                        cbName.setCurrentIndex(playerIdx)
         self.slotValidate()
 
-    def showEvent(self, unusedEvent:'QEvent') ->None:
+    def showEvent(self, unusedEvent:Optional['QEvent']) ->None:
         """start with player 0"""
         self.nameWidgets[0].setFocus()
 
@@ -142,8 +146,9 @@ class SelectPlayers(SelectRuleset):
                 combo.addItems(sorted(
                     allNames - self.__selectedNames() - {comboName}))
                 combo.setCurrentIndex(0)
-        self.buttonBox.button(QDialogButtonBox.StandardButton.Ok).setEnabled(
-            len(self.__selectedNames()) == 4)
+        ok_button = self.buttonBox.button(QDialogButtonBox.StandardButton.Ok)
+        if ok_button:
+            ok_button.setEnabled(len(self.__selectedNames()) == 4)
         self.names = [cbName.currentText() for cbName in self.nameWidgets]
 
 
@@ -151,11 +156,11 @@ class ScoringTileAttr(TileAttr):
 
     """Tile appearance is different in a ScoringHandBoard"""
 
-    def setDark(self) ->bool:
+    def _setDark(self) ->bool:
         """should the tile appear darker?"""
         return bool(self.yoffset) or self.tile.isConcealed
 
-    def setFocusable(self, hand:'HandBoard', meld:'Meld', idx:Optional[int]) ->bool:
+    def _setFocusable(self, hand:'HandBoard', meld:'Meld', idx:Optional[int]) ->bool:
         """in a scoring handboard, only the first tile of a meld is focusable"""
         return idx == 0
 
@@ -167,8 +172,8 @@ class ScoringHandBoard(HandBoard):
 
     def __init__(self, player:'ScoringPlayer') ->None:
         self.__moveHelper:Optional['QGraphicsItemGroup'] = None
-        self.uiMelds:List[UIMeld] = []
-        HandBoard.__init__(self, player)
+        self.uiMelds:List[UIMeld] = []  # uiMelds and uiTiles are administered independently
+        super().__init__(player)
         self.player:'ScoringPlayer'
 
     def meldVariants(self, tile:UITile, forLowerHalf:bool) ->MeldList:
@@ -194,12 +199,20 @@ class ScoringHandBoard(HandBoard):
         assert uiMeld
         return uiMeld[0]
 
-    def uiMeldWithTile(self, uiTile:UITile) ->UIMeld:
-        """return the meld with uiTile"""
-        for myMeld in self.uiMelds:
+    def uiMeldWithTile(self, uiTile:UITile, remove:bool=False) ->UIMeld:
+        """return the meld with uiTile, optionally removing it"""
+        for idx, myMeld in enumerate(self.uiMelds):
             if uiTile in myMeld:
+                uiMeld = self.uiMelds[idx]
+                if remove:
+                    if Debug.uitiles:
+                        logDebug(f'{callers()}.{self.debug_name()}.uiMeldWithTile({uiTile}) removes {uiMeld}')
+                    del self.uiMelds[idx]
+                    self.player.removeMeld(uiMeld.meld)
+                    if Debug.uitiles:
+                        logDebug(f'{self.debug_name()}.loseMeld({uiTile}) removed {uiMeld.meld} from {self.player}')
+                    self.sync()
                 return myMeld
-        logWarning(f'ScoringBoard: found no meld with {uiTile}')
         return UIMeld(uiTile)
 
     def findUIMeld(self, meld:Optional['Meld']) ->UIMeld:
@@ -209,29 +222,42 @@ class ScoringHandBoard(HandBoard):
                 return result
         logWarning(f'Scoring game: cannot find UIMeld for {meld}')
         if self.uiMelds:
-            return self.uiMelds[1]
+            # when could this happen?
+            logWarning(f'XXXXXXXXXXXXXX findUIMeld returning uiMelds[1]: {self.uiMelds[1]}')
+            return self.uiMelds[1] # FIXME why not 0 ?
         raise ValueError('Scoring Game: findUIMeld() in absence of any uiMelds')
 
-    def assignUITiles(self, uiTile:UITile, meld:'Meld') ->Optional[UIMeld]:  # pylint: disable=unused-argument
-        """generate a UIMeld. First uiTile is given, the rest should be as defined by meld"""
+    def loseMeld(self, uiTile:UITile, meld:'Meld') ->UIMeld:  # pylint: disable=unused-argument
+        """loses a UIMeld. Only first uiTile is given"""
         assert isinstance(uiTile, UITile), uiTile
-        return self.uiMeldWithTile(uiTile)
+        uiMeld = self.uiMeldWithTile(uiTile, remove=True)
+        scoringScene().handSelectorChanged(self)
+        return uiMeld
 
     def sync(self, adding:Optional[List[UITile]]=None) ->None:
         """place all tiles in ScoringHandBoard"""
-        self.placeTiles(cast(List[UITile], list(chain(*self.uiMelds))))
+        col = [0.0, 0.0]
+        boni:List[UITile] = []
+        for uiMeld in self.uiMelds:
+            if uiMeld[0].tile.isBonus:
+                boni.append(uiMeld[0])
+                continue
+            row = 0 if uiMeld.meld[0].isExposed else 1
+            for idx, tile in enumerate(uiMeld):
+                tile.level = 0
+                tile.setBoard(self, col[row], row)
+                # FIXME Folgendes UIMeld sollte sich selber so einstellen
+                tile.focusable = idx == 0
+                tile.dark = bool(row) or tile.tile.isConcealed
+                col[row] += 1
+            col[row] += self.exposedMeldDistance
+        self._placeBonusTiles(col, boni)
+        # self.checkTiles() does not work here
 
-    def deselect(self, meld:UIMeld) ->None:
-        """remove meld from old board"""
-        for idx, uiMeld in enumerate(self.uiMelds):
-            if all(id(meld[x]) == id(uiMeld[x]) for x in range(len(meld))):
-                del self.uiMelds[
-                    idx]  # do not use uiMelds.remove: If we have 2
-                break                 # identical melds, it removes the wrong one
-        self.player.removeMeld(meld)  # uiMeld must already be deleted
-
-    def dragMoveEvent(self, event:'QGraphicsSceneDragDropEvent') ->None:
+    def dragMoveEvent(self, event:Optional['QGraphicsSceneDragDropEvent']) ->None:
         """allow dropping of uiTile from ourself only to other state (open/concealed)"""
+        if not event:
+            return
         uiTile = cast('MimeData', event.mimeData()).uiTile
         localY = self.mapFromScene(QPointF(event.scenePos())).y()
         centerY = self.rect().height() / 2.0
@@ -250,43 +276,43 @@ class ScoringHandBoard(HandBoard):
             doAccept = oldLowerHalf != newLowerHalf
         event.setAccepted(doAccept)
 
-    def dropEvent(self, event:'QGraphicsSceneDragDropEvent') ->None:
+    def dropEvent(self, event:Optional['QGraphicsSceneDragDropEvent']) ->None:
         """drop into this handboard"""
+        if not event:
+            return
         uiTile = cast('MimeData', event.mimeData()).uiTile
         forLowerHalf = self.mapFromScene(
             QPointF(event.scenePos())).y() >= self.rect().height() / 2.0
-        if self.dropTile(uiTile, forLowerHalf):
+        if self.dropMeld(uiTile, forLowerHalf):
             event.accept()
         else:
             event.ignore()
         self._noPen()
 
-    def dropTile(self, uiTile:UITile, forLowerHalf:bool) ->bool:
-        """drop uiTile into lower or upper half of our hand"""
+    def dropMeld(self, uiTile:UITile, forLowerHalf:bool) ->bool:
+        """drop UIMeld containing uiTile into lower or upper half of our hand"""
         senderBoard = cast('SelectorBoard', uiTile.board)
         assert senderBoard
         newMeld = senderBoard.chooseVariant(uiTile, forLowerHalf)
         if not newMeld:
             return False
-        uiMeld = senderBoard.assignUITiles(uiTile, newMeld)
+        uiMeld = senderBoard.loseMeld(uiTile, newMeld)
         for uitile, tile in zip(uiMeld, newMeld):
             uitile.change_name(tile)
-        return self.dropMeld(uiMeld)
-
-    def dropMeld(self, uiMeld:UIMeld) ->bool:
-        """drop uiMeld into our hand"""
-        senderBoard = uiMeld[0].board
-        senderBoard.deselect(uiMeld)
         self.uiMelds.append(uiMeld)
+        if Debug.uitiles:
+            logDebug(f'{self.debug_name()}.dropMeld({uiTile}) appends{uiMeld}')
         assert self.player
         self.player.addMeld(uiMeld.meld)
+        if Debug.uitiles:
+            logDebug(f'{self.debug_name()}.dropMeld({uiTile}) added {uiMeld.meld} to {self.player}')
         self.sync()
-        self.hasLogicalFocus = senderBoard == self or not senderBoard.uiTiles
+        self.hasLogicalFocus = senderBoard == self or senderBoard.empty
         self.checkTiles()
         senderBoard.autoSelectTile()
         senderBoard.checkTiles()
         if senderBoard is not self and senderBoard.isHandBoard:
-            scoringScene().handSelectorChanged(senderBoard)
+            scoringScene().handSelectorChanged(cast('HandBoard', senderBoard))
         scoringScene().handSelectorChanged(self)
         animate()
         self.checkTiles()
@@ -297,26 +323,28 @@ class ScoringHandBoard(HandBoard):
         the entire meld"""
         focus = self.focusTile
         if not focus:
-            logWarning('ScoringBoard.focusRectWidth: there is no focus tile')
             return 1
         meld = self.uiMeldWithTile(focus)
         if not meld:
             logWarning(f'ScoringBoard.focusRectWidth: there is no meld with focus tile {focus}')
             return 1
+        if Debug.focusable:
+            logDebug(f'{callers()}.{self}.focusRectWidth for focusTile {self.focusTile} '
+                     f'found UIMeld {meld} with tiles {list(x for x in meld)}')
         return len(meld)
 
     def addUITile(self, uiTile:UITile) ->None:
-        Board.addUITile(self, uiTile)
+        super().addUITile(uiTile)
         self.showMoveHelper()
 
     def removeUITile(self, uiTile:UITile) ->None:
-        Board.removeUITile(self, uiTile)
+        super().removeUITile(uiTile)
         self.showMoveHelper()
 
     def showMoveHelper(self, visible:Optional[bool]=None) ->None:
         """show help text In empty HandBoards"""
         if visible is None:
-            visible = not self.uiTiles
+            visible = self.empty
         if self.__moveHelper and not isAlive(self.__moveHelper):
             return
         if visible:
@@ -344,17 +372,15 @@ class ScoringHandBoard(HandBoard):
                     if sceneRotation(self) == 180:
                         rotateCenter(helper, 180)
                     helpItems.append(helper)
-                self.__moveHelper = self.scene().createItemGroup(helpItems)
-            self.__moveHelper.setVisible(True)
+                scene = self.scene()
+                if scene:
+                    self.__moveHelper = scene.createItemGroup(helpItems)
+            if self.__moveHelper:
+                self.__moveHelper.setVisible(True)
         else:
             if self.__moveHelper:
                 self.__moveHelper.setVisible(False)
 
-    def newLowerMelds(self) ->MeldList:
-        """a list of melds for the hand as it should look after sync"""
-        assert self.player
-        return MeldList(self.player.concealedMelds)
-
 
 class ScoringPlayer(VisiblePlayer, Player):
 
@@ -370,7 +396,7 @@ class ScoringPlayer(VisiblePlayer, Player):
 
     def clearHand(self) ->None:
         """clears attributes related to current hand"""
-        Player.clearHand(self)
+        super().clearHand()
         if self.game and self.game.wall:
             assert self.game
             assert self.game.wall
@@ -477,13 +503,12 @@ class ScoringPlayer(VisiblePlayer, Player):
 
     def sortRulesByX(self, rules:List['UsedRule']) ->List['UsedRule']:
         """if this game has a GUI, sort rules by GUI order of the melds they are applied to"""
-        withMelds = [x for x in rules if x.meld]
-        withoutMelds = [x for x in rules if x not in withMelds]
-        assert (_ := cast('ScoringHandBoard', self.handBoard))
-        tuples = cast(Generator[Tuple['UsedRule', 'UIMeld'], None, None],
-            [tuple([x, _.findUIMeld(x.meld)]) for x in withMelds])
+        rulesWithMelds = [x for x in rules if x.meld]
+        rulesWithoutMelds = [x for x in rules if x not in rulesWithMelds]
+        assert (handBoard := cast('ScoringHandBoard', self.handBoard))
+        tuples:Any = [tuple([x, handBoard.findUIMeld(x.meld)]) for x in rulesWithMelds]
         sorted_tuples = sorted(tuples, key=lambda x: x[1][0].sortKey())
-        return [x[0] for x in sorted_tuples] + withoutMelds
+        return [x[0] for x in sorted_tuples] + rulesWithoutMelds
 
     def addMeld(self, meld:'Meld') ->None:
         """add meld to this hand in a scoring game"""
@@ -501,9 +526,8 @@ class ScoringPlayer(VisiblePlayer, Player):
                 logDebug(f'{self} gets exposed meld {meld}')
         self._hand = None
 
-    def removeMeld(self, uiMeld:UIMeld) ->None:
+    def removeMeld(self, meld:'Meld') ->None:
         """remove a meld from this hand in a scoring game"""
-        meld = uiMeld.meld
         if meld.isBonus:
             self._bonusTiles.remove(meld[0])
             if Debug.scoring:
@@ -535,8 +559,7 @@ class ScoringGame(Game):
 
     def __init__(self, names:List[Tuple[Wind, str]], ruleset:'Ruleset', gameid:Optional[int]=None,
         wantedGame:Optional[str]=None, client:Optional['Client']=None) ->None:
-        Game.__init__(
-            self,
+        super().__init__(
             names,
             ruleset,
             gameid=gameid,
@@ -597,7 +620,7 @@ class ScoringGame(Game):
         if self.wall:
             self.wall.hide()
         SideText.removeAll()
-        return Game.close(self)
+        return super().close()
 
     @staticmethod
     def isScoringGame() ->bool:
@@ -619,7 +642,7 @@ class ScoringGame(Game):
         """get a new id"""
         if not self.gameid:
             # a loaded game has gameid already set
-            self.gameid = self._newGameId()
+            self.gameid = Query("insert into game(seed) values(0)").cursor.lastrowid
 
     def _mustExchangeSeats(self, pairs:List[List[Player]]) ->List[List[Player]]:
         """filter: which player pairs should really swap places?"""
@@ -635,16 +658,15 @@ class ScoringGame(Game):
         Query("INSERT INTO SCORE "
               "(game,penalty,hand,data,manualrules,player,scoretime,"
               "won,prevailing,wind,points,payments, balance,rotated,notrotated) "
-              f"VALUES({int(self.gameid)},1,{int(self.handctr)},?,?,{player.nameid},'{scoretime}',"
-              f"{int(player == self.winner)},'{self.roundWind}','{player.wind}',0,{amount},"
-              f"{player.balance},{int(self.rotated)}, {int(self.notRotated)})",
+              f"VALUES({int(self.gameid)},1,{int(self.point.handCount)},?,?,{player.nameid},'{scoretime}',"
+              f"{int(player == self.winner)},'{self.point.prevailing}','{player.wind}',0,{amount},"
+              f"{player.balance},{int(self.point.rotated)}, {int(self.point.notRotated)})",
               (player.hand.string, offense.name))
         assert Internal.mainWindow
         Internal.mainWindow.updateGUI()
 
 def scoreGame() ->Optional[ScoringGame]:
     """show all games, select an existing game or create a new game"""
-    openDb()
     Players.load()
     if len(Players.humanNames) < 4:
         logWarning(
