@@ -3,7 +3,7 @@
 """
 Copyright (C) 2008-2016 Wolfgang Rohdewald <wolfgang@rohdewald.de>
 
-SPDX-License-Identifier: GPL-2.0
+SPDX-License-Identifier: GPL-2.0-only
 
 """
 
@@ -20,6 +20,7 @@ from qt import QAbstractItemView, QHeaderView
 from qt import QTreeView, QFont, QFrame
 from qt import QStyledItemDelegate
 from qt import QBrush, QPalette
+from qt import PYQT6
 from kde import KDialogButtonBox, KApplication
 
 from modeltest import ModelTest
@@ -32,8 +33,8 @@ from statesaver import StateSaver
 from query import Query
 from guiutil import ListComboBox, Painter, decorateWindow, BlockSignals
 from tree import TreeItem, RootItem, TreeModel
-from wind import Wind
 from tile import Tile, MeldList
+from point import Point
 
 if TYPE_CHECKING:
     from qt import QObject, QRect, QStyleOptionViewItem, QPersistentModelIndex
@@ -79,7 +80,7 @@ class ScoreGroupItem(ScoreTreeItem):
     """represents a group in the tree like Points, Payments, Balance"""
 
     def __init__(self, content:str) ->None:
-        ScoreTreeItem.__init__(self, content)
+        super().__init__(content)
 
     def content(self, column:int) ->str:
         """return content stored in this item"""
@@ -90,10 +91,10 @@ class ScorePlayerItem(ScoreTreeItem):
 
     """represents a player in the tree"""
 
-    def __init__(self, content:Tuple[str, List['HandResult']]) ->None:
-        ScoreTreeItem.__init__(self, content)
+    def __init__(self, content:Tuple[str, List[Any]]) ->None:
+        super().__init__(content)
 
-    def content(self, column:int) ->Union[str, 'HandResult', None]:
+    def content(self, column:int) ->Union[str, Any, None]:
         """return the content stored in this node"""
         if column == 0:
             return i18n(self.raw[0])
@@ -104,7 +105,7 @@ class ScorePlayerItem(ScoreTreeItem):
             # not happen in practical use
             return None
 
-    def hands(self) ->List['HandResult']:
+    def hands(self) ->List[Any]:
         """a small helper"""
         return self.raw[1]
 
@@ -146,16 +147,18 @@ class ScoreItemDelegate(QStyledItemDelegate):
     colors.append(QColor('orange'))
 
     def __init__(self, parent:Optional['QObject']=None) ->None:
-        QStyledItemDelegate.__init__(self, parent)
+        super().__init__(parent)
 
-    def paint(self, painter:QPainter, option:'QStyleOptionViewItem',
+    def paint(self, painter:Optional[QPainter], option:'QStyleOptionViewItem',
         index:Union[QModelIndex,'QPersistentModelIndex']) ->None:
         """where the real work is done..."""
-        assert isinstance(index, ScorePlayerItem)
+        assert painter
+        assert isinstance(index, QModelIndex), index
         item = index.internalPointer()
         if isinstance(item, ScorePlayerItem) and item.parent and item.parent.row() == 3 and index.column() != 0:
             parent_item = cast(TreeItem, index.parent().internalPointer())
             for idx, playerItem in enumerate(parent_item.children):
+                assert isinstance(playerItem, ScorePlayerItem), playerItem
                 rect = option.rect  # type:ignore[attr-defined]
                 chart = cast('ScoreModel', index.model()).chart(rect, index, playerItem)
                 if chart:
@@ -167,9 +170,13 @@ class ScoreItemDelegate(QStyledItemDelegate):
                         # separately per cell beause the lines spread vertically over two rows: We would
                         # have to draw the lines into one big pixmap and copy
                         # from the into the cells
-                        painter.drawPolyline(chart)
+                        if PYQT6:
+                            # seems the annotations are wrong: list(QPointF) does work
+                            painter.drawPolyline(chart)  # type:ignore[call-overload]
+                        else:
+                            painter.drawPolyline(chart)
             return
-        QStyledItemDelegate.paint(self, painter, option, index)
+        super().paint(painter, option, index)
 
 
 class ScoreModel(TreeModel):
@@ -177,6 +184,8 @@ class ScoreModel(TreeModel):
     """a model for our score table"""
     steps = 30  # how fine do we want the stepping in the chart spline
 
+    tupleType: Any = type(None)
+
     def __init__(self, scoreTable:'ScoreTable', parent:Optional['QObject']=None) ->None:
         super().__init__(parent)
         self.scoreTable = scoreTable
@@ -208,7 +217,8 @@ class ScoreModel(TreeModel):
         if role == Qt.ItemDataRole.DisplayRole:
             if isinstance(item, ScorePlayerItem):
                 content = item.content(column)
-                if isinstance(content, HandResult):
+                # if content.__class__.__name__ == 'Score':
+                if isinstance(content, ScoreModel.tupleType):
                     parentRow = item.parent.row()
                     if parentRow == 0:
                         if not content.penalty:
@@ -227,15 +237,16 @@ class ScoreModel(TreeModel):
         if role == Qt.ItemDataRole.ForegroundRole:
             if isinstance(item, ScorePlayerItem) and item.parent.row() == 3:
                 content = item.content(column)
-                if not isinstance(content, HandResult):
+                if not isinstance(content, ScoreModel.tupleType):
                     return QBrush(ScoreItemDelegate.colors[index.row()])
         if column > 0 and isinstance(item, ScorePlayerItem):
-            content = cast('HandResult', item.content(column))
+            content = item.content(column)  # type:ignore
             if role == Qt.ItemDataRole.BackgroundRole:
+                assert isinstance(content, ScoreModel.tupleType)
                 if content and content.won:
                     return QColor(165, 255, 165)
             if role == Qt.ItemDataRole.ToolTipRole:
-                englishHints = content.manualrules.split('||')
+                englishHints = content.manualrules.split('||')  # type:ignore
                 tooltip = '<br />'.join(i18n(x) for x in englishHints)
                 return tooltip
         return None
@@ -252,7 +263,7 @@ class ScoreModel(TreeModel):
                 hands = child1.hands()
                 handResult = hands[section - 1]
                 if not handResult.penalty:
-                    return handResult.handId()
+                    return self.handTitle(handResult)
         elif role == Qt.ItemDataRole.TextAlignmentRole:
             return int(Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter) \
                 if section == 0 else int(Qt.AlignmentFlag.AlignRight | Qt.AlignmentFlag.AlignVCenter)
@@ -263,16 +274,24 @@ class ScoreModel(TreeModel):
         game = self.scoreTable.game
         assert game
         assert game.gameid
-        records = Query(
-            'select player,rotated,notrotated,penalty,won,prevailing,wind,points,payments,balance,manualrules'
-            ' from score where game=? order by player,hand', (game.gameid,)).records
+
+        fields = 'game,hand,player,rotated,notrotated,penalty,won,prevailing,wind,points,payments,balance,manualrules'
+        tuples = Query('select {fields} from score where game=? order by hand', (game.gameid, ),
+            fields=fields).tuples()
+#        if not tuples:
+#            return
+        if tuples:
+            ScoreModel.tupleType = tuples[0].__class__
         humans = sorted(
             (x for x in game.players if not x.name.startswith('Robot')))
         robots = sorted(
             (x for x in game.players if x.name.startswith('Robot')))
-        data =  cast(List[Tuple[str, List['HandResult']]],
-                    [tuple([player.localName, [HandResult(*x[1:]) for x in records
-                    if x[0] == player.nameid]]) for player in humans + robots])
+        data =  cast(List[Tuple[str, List[Any]]],
+                    [tuple([player.localName, [x for x in tuples  # type:ignore
+                    if x.player == player.nameid]]) for player in humans + robots])
+#        print(f'players: {Players.allNames}')
+#        for idx, _ in enumerate(data):
+#            print(f'{idx}: {_}')
         self.__findMinMaxChartPoints(data)
         parent = QModelIndex()
         assert self.rootItem
@@ -285,7 +304,7 @@ class ScoreModel(TreeModel):
             for idx1, item in enumerate(data):
                 self.insertRows(idx1, list([ScorePlayerItem(item)]), listIndex)
 
-    def __findMinMaxChartPoints(self, data:List[Tuple[str, List['HandResult']]]) ->None:
+    def __findMinMaxChartPoints(self, data:List[Tuple[str, List[Any]]]) ->None:
         """find and save the extremes of the spline. They can be higher than
         the pure balance values"""
         self.minY = 9999999.9
@@ -299,47 +318,9 @@ class ScoreModel(TreeModel):
         self.minY -= 2  # antialiasing might cross the cell border
         self.maxY += 2
 
-
-class HandResult:
-
-    """holds the results of a hand for the scoring table"""
-    # pylint: disable=too-many-arguments
-    # we have too many arguments
-
-    def __init__(self, rotated:int, notRotated:int, penalty:bool, won:bool,
-                 prevailing:str, wind:str, points:int, payments:int, balance:int, manualrules:str) ->None:
-        self.rotated = rotated
-        self.notRotated = notRotated
-        self.penalty = penalty
-        self.won = won
-        self.prevailing = Wind(prevailing)
-        self.wind = Wind(wind)
-        self.points = points
-        self.payments = payments
-        self.balance = balance
-        self.manualrules = manualrules
-
-    def __str__(self) ->str:
-        return (f'{int(self.penalty)} {int(self.points)} {self.wind} '
-                f'{int(self.payments)} {int(self.balance)} {self.manualrules}')
-
-    def handId(self) ->str:
+    def handTitle(self, handResult:Any) ->str:
         """identifies the hand for window title and scoring table"""
-        character = chr(
-            ord('a') - 1 + self.notRotated) if self.notRotated else ''
-        return f'{self.prevailing}{self.rotated + 1}{character}'
-
-    def roundHand(self, allHands:List['HandResult']) ->int:
-        """the nth hand in the current round, starting with 1"""
-        idx = allHands.index(self)
-        _ = reversed(allHands[:idx])
-        allHands = [x for x in _ if not x.penalty]
-        if not allHands:
-            return 1
-        for idx, hand in enumerate(allHands):
-            if hand.prevailing != self.prevailing:
-                return idx + 1
-        return idx + 2
+        return str(Point(handResult))
 
 
 class ScoreViewLeft(QTreeView):
@@ -347,7 +328,7 @@ class ScoreViewLeft(QTreeView):
     """subclass for defining sizeHint"""
 
     def __init__(self, parent:Optional[QWidget]=None) ->None:
-        QTreeView.__init__(self, parent)
+        super().__init__(parent)
         self.setItemDelegate(ScoreItemDelegate(self))
 
     def __col0Width(self) ->int:
@@ -371,23 +352,25 @@ class ScoreViewRight(QTreeView):
     """we need to subclass for catching events"""
 
     def __init__(self, parent:Optional[QWidget]=None) ->None:
-        QTreeView.__init__(self, parent)
+        super().__init__(parent)
         self.setItemDelegate(ScoreItemDelegate(self))
 
-    def changeEvent(self, event:QEvent) ->None:
+    def changeEvent(self, event:Optional[QEvent]) ->None:
         """recompute column width if font changes"""
-        if event.type() == QEvent.Type.FontChange:
-            self.setColWidth()
+        if event:
+            if event.type() == QEvent.Type.FontChange:
+                self.setColWidth()
 
     def setColWidth(self) ->None:
         """we want a fixed column width sufficient for all values"""
-        colRange = range(1, self.header().count())
-        if colRange:
-            for col in colRange:
-                self.resizeColumnToContents(col)
-            width = max(self.columnWidth(x) for x in colRange)
-            for col in colRange:
-                self.setColumnWidth(col, width)
+        if header := self.header():
+            colRange = range(1, header.count())
+            if colRange:
+                for col in colRange:
+                    self.resizeColumnToContents(col)
+                width = max(self.columnWidth(x) for x in colRange)
+                for col in colRange:
+                    self.setColumnWidth(col, width)
 
 
 class HorizontalScrollBar(QScrollBar):
@@ -395,17 +378,17 @@ class HorizontalScrollBar(QScrollBar):
     """We subclass here because we want to react on show/hide"""
 
     def __init__(self, scoreTable:'ScoreTable', parent:Optional[QWidget]=None) ->None:
-        QScrollBar.__init__(self, parent)
+        super().__init__(parent)
         self.scoreTable = scoreTable
 
-    def showEvent(self, unusedEvent:QEvent) ->None:
+    def showEvent(self, unusedEvent:Optional[QEvent]) ->None:
         """adjust the left view"""
         self.scoreTable.adaptLeftViewHeight()
 
-    def hideEvent(self, unusedEvent:QEvent) ->None:
+    def hideEvent(self, unusedEvent:Optional[QEvent]) ->None:
         """adjust the left view"""
-        self.scoreTable.viewRight.header().setOffset(
-            0)  # we should not have to do this...
+        if header := self.scoreTable.viewRight.header():
+            header.setOffset(0)  # we should not have to do this...
         # how to reproduce problem without setOffset:
         # show table with hor scroll, scroll to right, extend window
         # width very fast. The faster we do that, the wronger the
@@ -426,6 +409,7 @@ class ScoreTable(QWidget):
         self.scoreModelTest:Optional[ModelTest] = None
         decorateWindow(self, i18nc('kajongg', 'Scores'))
         self.setAttribute(Qt.WidgetAttribute.WA_AlwaysShowToolTips)
+        self.setWindowFlags(self.windowFlags() | Qt.WindowType.Tool)
         self.setMouseTracking(True)
         self.setupUi()
         self.refresh()
@@ -449,8 +433,9 @@ class ScoreTable(QWidget):
         self.viewRight.setHorizontalScrollBar(HorizontalScrollBar(self))
         self.viewRight.setHorizontalScrollMode(QAbstractItemView.ScrollMode.ScrollPerItem)
         self.viewRight.setFocusPolicy(Qt.FocusPolicy.NoFocus)
-        self.viewRight.header().setSectionsClickable(False)
-        self.viewRight.header().setSectionsMovable(False)
+        if header := self.viewRight.header():
+            header.setSectionsClickable(False)
+            header.setSectionsMovable(False)
         self.viewRight.setSelectionMode(QAbstractItemView.SelectionMode.NoSelection)
         windowLayout = QVBoxLayout(self)
         self.splitter = QSplitter(Qt.Orientation.Vertical)
@@ -486,6 +471,7 @@ class ScoreTable(QWidget):
     def refresh(self) ->None:
         """load this game and this player. Keep parameter list identical with
         ExplainView"""
+        # pylint:disable=too-many-branches
         if not self.game:
             # keep scores of previous game on display
             return
@@ -508,13 +494,17 @@ class ScoreTable(QWidget):
             self.scoreModelTest = ModelTest(self.scoreModel, self)
         for view in [self.viewLeft, self.viewRight]:
             view.setModel(self.scoreModel)
-            header = view.header()
-            header.setStretchLastSection(False)
+            if header := view.header():
+                header.setStretchLastSection(False)
             view.setAlternatingRowColors(True)
-        self.viewRight.header().setSectionResizeMode(QHeaderView.ResizeMode.Fixed)
-        for col in range(self.viewLeft.header().count()):
-            self.viewLeft.header().setSectionHidden(col, col > 0)
-            self.viewRight.header().setSectionHidden(col, col == 0)
+        left_header = self.viewLeft.header()
+        right_header = self.viewRight.header()
+        if not left_header or not right_header:
+            return
+        right_header.setSectionResizeMode(QHeaderView.ResizeMode.Fixed)
+        for col in range(left_header.count()):
+            left_header.setSectionHidden(col, col > 0)
+            right_header.setSectionHidden(col, col == 0)
         self.scoreLayout.setStretch(1, 100)
         self.scoreLayout.setSpacing(0)
         self.viewLeft.setFrameStyle(QFrame.Shape.NoFrame)
@@ -523,8 +513,10 @@ class ScoreTable(QWidget):
         for master, slave in ((self.viewRight, self.viewLeft), (self.viewLeft, self.viewRight)):
             master.expanded.connect(slave.expand)
             master.collapsed.connect(slave.collapse)
-            master.verticalScrollBar().valueChanged.connect(
-                slave.verticalScrollBar().setValue)
+            master_sb = master.verticalScrollBar()
+            slave_sb = slave.verticalScrollBar()
+            if master_sb and slave_sb:
+                master_sb.valueChanged.connect(slave_sb.setValue)
         for row, expand in enumerate(expandGroups):
             self.viewLeft.setExpanded(
                 self.scoreModel.index(row,
@@ -538,10 +530,10 @@ class ScoreTable(QWidget):
 
     def scrollRight(self) ->None:
         """make sure the latest hand is visible"""
-        scrollBar = self.viewRight.horizontalScrollBar()
-        scrollBar.setValue(scrollBar.maximum())
+        if scrollBar := self.viewRight.horizontalScrollBar():
+            scrollBar.setValue(scrollBar.maximum())
 
-    def showEvent(self, unusedEvent:QEvent) ->None:
+    def showEvent(self, unusedEvent:Optional[QEvent]) ->None:
         """Only now the views and scrollbars have useful sizes, so we can compute the spacer
         for the left view"""
         self.adaptLeftViewHeight()
@@ -550,8 +542,9 @@ class ScoreTable(QWidget):
         """if the right view has a horizontal scrollbar, make sure both
         view have the same vertical scroll area. Otherwise scrolling to
         bottom results in unsyncronized views."""
-        if self.viewRight.horizontalScrollBar().isVisible():
-            height = self.viewRight.horizontalScrollBar().height()
+        h_bar = self.viewRight.horizontalScrollBar()
+        if h_bar and h_bar.isVisible():
+            height = h_bar.height()
         else:
             height = 0
         if self.leftLayout.count() > 1:
@@ -560,7 +553,7 @@ class ScoreTable(QWidget):
         if height:
             self.leftLayout.addSpacing(height)
 
-    def closeEvent(self, unusedEvent:QEvent) ->None:
+    def closeEvent(self, unusedEvent:Optional[QEvent]) ->None:
         """update action button state"""
         assert Internal.mainWindow
         Internal.mainWindow.actionScoreTable.setChecked(False)
@@ -616,7 +609,7 @@ class ExplainView(QListView):
             # QStringListModel does not optimize identical lists away, so we do
             self._model.setStringList(lines)
 
-    def closeEvent(self, unusedEvent:QEvent) ->None:
+    def closeEvent(self, unusedEvent:Optional[QEvent]) ->None:
         """update action button state"""
         assert Internal.mainWindow
         Internal.mainWindow.actionExplain.setChecked(False)
@@ -627,13 +620,13 @@ class PenaltyBox(QSpinBox):
     """with its own validator, we only accept multiples of parties"""
 
     def __init__(self, parties:int, parent:Optional[QWidget]=None) ->None:
-        QSpinBox.__init__(self, parent)
+        super().__init__(parent)
         self.parties = parties
         self.prevValue = 0
 
-    def validate(self, inputData:str, pos:int) -> Tuple[QValidator.State, str, int]:
+    def validate(self, inputData:str, pos:int) -> Tuple[QValidator.State, str, int]:  # type:ignore[override]
         """check if value is a multiple of parties"""
-        _ = QSpinBox.validate(self, inputData, pos)
+        _ = super().validate(inputData, pos)
         result = _[0]
         inputData = _[1]
         newPos = _[2]
@@ -648,7 +641,7 @@ class PenaltyBox(QSpinBox):
             self.prevValue = int(inputData)
         return (result, inputData, newPos)
 
-    def fixup(self, data:str) ->str:
+    def fixup(self, data:str) ->str:  # type:ignore[override]
         """change input to a legal value"""
         # The Qt doc says return type is None but annotation says str
         value = int(str(data))
@@ -668,7 +661,7 @@ class RuleBox(QCheckBox):
     """additional attribute: ruleId"""
 
     def __init__(self, rule:'Rule') ->None:
-        QCheckBox.__init__(self, i18n(rule.name))
+        super().__init__(i18n(rule.name))
         self.rule = rule
 
     def setApplicable(self, applicable:bool) ->None:
@@ -692,7 +685,7 @@ class PenaltyDialog(QDialog):
 
     def __init__(self, game:'ScoringGame', parent:Optional[QWidget]=None) ->None:
         """selection for this player, tiles are the still available tiles"""
-        QDialog.__init__(self, parent)
+        super().__init__(parent)
         decorateWindow(self, i18n("Penalty"))
         self.game = game
         grid = QGridLayout(self)
@@ -733,6 +726,7 @@ class PenaltyDialog(QDialog):
         self.btnExecute = buttonBox.addButton(
             i18n("&Execute"),
             QDialogButtonBox.ButtonRole.AcceptRole)
+        assert self.btnExecute
         self.btnExecute.clicked.connect(self.accept)
         self.crimeChanged()
         StateSaver(self)
@@ -822,7 +816,7 @@ class ScoringDialog(QWidget):
     # pylint: disable=too-many-instance-attributes
 
     def __init__(self, scene:'ScoringScene', parent:Optional[QWidget]=None) ->None:
-        QWidget.__init__(self, parent)
+        super().__init__(parent)
         self.scene = scene
         decorateWindow(self, i18nc("@title:window", "Scoring for this Hand"))
         self.nameLabels = list(QLabel() for x in range(4))
@@ -991,12 +985,13 @@ class ScoringDialog(QWidget):
     def wonChanged(self) ->None:
         """if a new winner has been defined, uncheck any previous winner"""
         newWinner = None
-        if self.sender() != self.draw:
-            clicked = self.clickedPlayerIdx(self.sender())
-            if self.wonBoxes[clicked].isChecked():
-                newWinner = self.game.players[clicked]
-            else:
-                newWinner = None
+        if sender := self.sender():
+            if sender != self.draw:
+                clicked = self.clickedPlayerIdx(sender)
+                if self.wonBoxes[clicked].isChecked():
+                    newWinner = self.game.players[clicked]
+                else:
+                    newWinner = None
         self.game.winner = newWinner
         for idx in range(4):
             if newWinner != self.game.players[idx]:
@@ -1021,10 +1016,9 @@ class ScoringDialog(QWidget):
                     for pBox in player.manualRuleBoxes:
                         if pBox.rule.name == ruleBox.rule.name:
                             pBox.setChecked(False)
-        try:
-            newState = bool(self.game.winner.handBoard.uiTiles) # type:ignore[union-attr]
-        except AttributeError:
-            newState = False
+        newState = False
+        if self.game.winner and self.game.winner.handBoard:
+            newState = self.game.winner.handBoard.empty
         self.lblLastTile.setVisible(newState)
         self.cbLastTile.setVisible(newState)
         if self.game:
@@ -1057,7 +1051,7 @@ class ScoringDialog(QWidget):
         """update their wind and prevailing"""
         for idx, player in enumerate(self.game.players):
             self.windLabels[idx].wind = player.wind
-            self.windLabels[idx].roundsFinished = self.game.roundsFinished
+            self.windLabels[idx].prevailing = self.game.point.prevailing
 
     def computeScores(self) ->None:
         """if tiles have been selected, compute their value"""
@@ -1071,7 +1065,7 @@ class ScoringDialog(QWidget):
                 self.nameLabels, self.wonBoxes, self.spValues, self.game.players):
             with BlockSignals([spValue, wonBox]):
                 # we do not want that change to call computeScores again
-                if player.handBoard and player.handBoard.uiTiles:
+                if player.handBoard and not player.handBoard.empty:
                     spValue.setEnabled(False)
                     nameLabel.setBuddy(wonBox)
                     for _ in range(10):
@@ -1126,10 +1120,7 @@ class ScoringDialog(QWidget):
         if not winnerTiles:
             return
         assert winnerTiles[0].board
-        # mypy does not seem to understand operator*
-        _ = winnerTiles[0].board.tileset.faceSize
-        _ *= 0.5
-        pmSize = _.toSize()
+        pmSize = winnerTiles[0].board.tileset.faceSize.toSize()
         self.cbLastTile.setIconSize(pmSize)
         QPixmapCache.clear()
         shownTiles = set()
@@ -1177,8 +1168,8 @@ class ScoringDialog(QWidget):
         winner = self.game.winner
         assert winner
         assert winner.handBoard
-        faceWidth = int(winner.handBoard.tileset.faceSize.width() * 0.5)
-        faceHeight = int(winner.handBoard.tileset.faceSize.height() * 0.5)
+        faceWidth = int(winner.handBoard.tileset.faceSize.width())
+        faceHeight = int(winner.handBoard.tileset.faceSize.height())
         restoredIdx = None
         for meld in winnerMelds:
             pixMap = QPixmap(faceWidth * len(meld), faceHeight)
@@ -1190,7 +1181,7 @@ class ScoringDialog(QWidget):
                                    winner.handBoard.tilesByElement(element)
                                    [0].pixmapFromSvg(QSize(faceWidth, faceHeight), withBorders=False))
                 painter.translate(QPointF(faceWidth, 0.0))
-            self.cbLastMeld.addItem(QIcon(pixMap), '', str(meld))
+            self.cbLastMeld.addItem(QIcon(pixMap), '', meld)
             if indexedMeld == meld:
                 restoredIdx = self.cbLastMeld.count() - 1
         if not restoredIdx and indexedMeld:
@@ -1198,7 +1189,7 @@ class ScoringDialog(QWidget):
             indexedMeld = indexedMeld.exposed
             for idx in range(self.cbLastMeld.count()):
                 meldContent = self.cbLastMeld.itemData(idx)
-                if indexedMeld == meldContent.lower():
+                if indexedMeld == meldContent.exposed:
                     restoredIdx = idx
                     if lastTile is not None and lastTile not in meldContent:
                         lastTile = lastTile.swapped
@@ -1232,7 +1223,7 @@ class ScoringDialog(QWidget):
                            and lastTile in m)
             assert winnerMelds, f'lastTile {lastTile} missing in {self.game.winner.hand.melds}'
             if len(winnerMelds) == 1:
-                self.cbLastMeld.addItem(QIcon(), '', str(winnerMelds[0]))
+                self.cbLastMeld.addItem(QIcon(), '', winnerMelds[0])
                 self.cbLastMeld.setCurrentIndex(0)
                 self.lblLastMeld.setVisible(False)
                 self.cbLastMeld.setVisible(False)
